@@ -80,6 +80,10 @@
 
 #define QT_FEATURE_arm_crypto -1
 
+#define QT_FEATURE_localtime_r 1
+
+#define QT_FEATURE_localtime_s -1
+
 #define QT_FEATURE_posix_fallocate 1
 
 #define QT_FEATURE_alloca_h 1
@@ -88,8 +92,6 @@
 
 #define QT_FEATURE_alloca 1
 
-#define QT_FEATURE_stack_protector_strong -1
-
 #define QT_FEATURE_system_zlib 1
 
 #define QT_FEATURE_stdlib_libcpp -1
@@ -122,3 +124,17 @@
 
 #define QT_FEATURE_intelcet -1
 
+#define QT_FEATURE_glibc_fortify_source 1
+
+#define QT_FEATURE_trivial_auto_var_init_pattern -1
+
+#define QT_FEATURE_stack_protector 1
+
+#define QT_FEATURE_stack_clash_protection 1
+
+#define QT_FEATURE_libstdcpp_assertions 1
+
+#define QT_FEATURE_libcpp_hardening -1
+
+#define QT_FEATURE_relro_now_linker 1
+
